@@ -19,10 +19,14 @@
  */
 package com.eteks.sweethome3d.j3d;
 
+import java.awt.Dimension;
 import java.awt.geom.Area;
 import java.awt.geom.PathIterator;
 import java.awt.geom.Point2D;
 import java.awt.geom.Rectangle2D;
+import java.awt.image.BufferedImage;
+import java.io.IOException;
+import java.io.InputStream;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
@@ -30,19 +34,32 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
+import javax.imageio.ImageIO;
 import javax.media.j3d.Appearance;
+import javax.media.j3d.BranchGroup;
+import javax.media.j3d.RenderingAttributes;
 import javax.media.j3d.Shape3D;
+import javax.media.j3d.TexCoordGeneration;
 import javax.media.j3d.Texture;
+import javax.media.j3d.TextureAttributes;
+import javax.media.j3d.Transform3D;
+import javax.media.j3d.TransformGroup;
 import javax.media.j3d.TransparencyAttributes;
 import javax.vecmath.Point3f;
 import javax.vecmath.TexCoord2f;
+import javax.vecmath.Vector3d;
+import javax.vecmath.Vector4f;
 
+import com.eteks.sweethome3d.model.BackgroundImage;
+import com.eteks.sweethome3d.model.Content;
 import com.eteks.sweethome3d.model.Home;
+import com.eteks.sweethome3d.model.HomeFurnitureGroup;
 import com.eteks.sweethome3d.model.HomePieceOfFurniture;
 import com.eteks.sweethome3d.model.HomeTexture;
 import com.eteks.sweethome3d.model.Level;
 import com.eteks.sweethome3d.model.Room;
 import com.eteks.sweethome3d.model.Wall;
+import com.sun.j3d.utils.geometry.Box;
 import com.sun.j3d.utils.geometry.GeometryInfo;
 import com.sun.j3d.utils.geometry.NormalGenerator;
 
@@ -51,11 +68,20 @@ import com.sun.j3d.utils.geometry.NormalGenerator;
  * @author Emmanuel Puybaret
  */
 public class Ground3D extends Object3DBranch {
+  private static final TextureAttributes MODULATE_TEXTURE_ATTRIBUTES = new TextureAttributes();
+
+  static {
+    MODULATE_TEXTURE_ATTRIBUTES.setTextureMode(TextureAttributes.MODULATE);
+  }
+
   private final float originX;
   private final float originY;
   private final float width;
   private final float depth;
 
+  private Content   backgroundImageCache;
+  private Dimension backgroundImageDimensionCache;
+
   /**
    * Creates a 3D ground for the given <code>home</code>.
    */
@@ -63,7 +89,7 @@ public class Ground3D extends Object3DBranch {
                   float originX,
                   float originY,
                   float width,
-                  float depth, 
+                  float depth,
                   boolean waitTextureLoadingEnd) {
     setUserData(home);
     this.originX = originX;
@@ -85,14 +111,35 @@ public class Ground3D extends Object3DBranch {
     groundShape.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
     groundShape.setCapability(Shape3D.ALLOW_GEOMETRY_READ);
     groundShape.setCapability(Shape3D.ALLOW_APPEARANCE_READ);
-    
-    setCapability(ALLOW_CHILDREN_READ);
-    
     addChild(groundShape);
 
-    update(waitTextureLoadingEnd);    
+    Appearance backgroundImageAppearance = new Appearance();
+    backgroundImageAppearance.setMaterial(getMaterial(DEFAULT_COLOR, DEFAULT_COLOR, 0));
+    backgroundImageAppearance.setTextureAttributes(MODULATE_TEXTURE_ATTRIBUTES);
+    backgroundImageAppearance.setTexCoordGeneration(new TexCoordGeneration(TexCoordGeneration.OBJECT_LINEAR,
+        TexCoordGeneration.TEXTURE_COORDINATE_2, new Vector4f(1, 0, 0, .5f), new Vector4f(0, 1, -1, .5f)));
+    backgroundImageAppearance.setCapability(Appearance.ALLOW_TEXTURE_WRITE);
+    RenderingAttributes renderingAttributes = new RenderingAttributes();
+    renderingAttributes.setCapability(RenderingAttributes.ALLOW_VISIBLE_WRITE);
+    backgroundImageAppearance.setRenderingAttributes(renderingAttributes);
+
+    TransformGroup transformGroup = new TransformGroup();
+    // Allow the change of the transformation that sets background image size and position
+    transformGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
+    transformGroup.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
+
+    Box box = new Box(0.5f, 0f, 0.5f, backgroundImageAppearance);
+    Shape3D backgroundImageShape = box.getShape(Box.TOP);
+    backgroundImageShape.setCapability(Shape3D.ALLOW_APPEARANCE_READ);
+    box.removeChild(backgroundImageShape);
+    transformGroup.addChild(backgroundImageShape);
+    addChild(transformGroup);
+
+    setCapability(ALLOW_CHILDREN_READ);
+
+    update(waitTextureLoadingEnd);
   }
-  
+
   /**
    * Updates ground coloring and texture attributes from home ground color and texture.
    */
@@ -100,22 +147,86 @@ public class Ground3D extends Object3DBranch {
   public void update() {
     update(false);
   }
-  
+
   /**
    * Updates the geometry and attributes of ground and sublevels.
    */
-  private void update(boolean waitTextureLoadingEnd) {
+  private void update(final boolean waitTextureLoadingEnd) {
     final Home home = (Home)getUserData();
+    // Update background image viewed on ground
+    final TransformGroup backgroundImageGroup = (TransformGroup)getChild(1);
+    Shape3D backgroundImageShape = (Shape3D)backgroundImageGroup.getChild(0);
+    final Appearance backgroundImageAppearance = backgroundImageShape.getAppearance();
+    RenderingAttributes backgroundImageRenderingAttributes = backgroundImageAppearance.getRenderingAttributes();
+    BackgroundImage backgroundImage = null;
+    if (home.getEnvironment().isBackgroundImageVisibleOnGround3D()) {
+      List<Level> levels = home.getLevels();
+      if (levels.size() > 0) {
+        for (int i = levels.size() - 1; i >= 0; i--) {
+          Level level = levels.get(i);
+          if (level.getElevation() == 0
+              && level.isViewableAndVisible()
+              && level.getBackgroundImage() != null
+              && level.getBackgroundImage().isVisible()) {
+            backgroundImage = level.getBackgroundImage();
+            break;
+          }
+        }
+      } else if (home.getBackgroundImage() != null
+                && home.getBackgroundImage().isVisible()) {
+        backgroundImage = home.getBackgroundImage();
+      }
+    }
+    if (backgroundImage != null) {
+      final BackgroundImage displayedBackgroundImage = backgroundImage;
+      TextureManager.getInstance().loadTexture(displayedBackgroundImage.getImage(), waitTextureLoadingEnd,
+          new TextureManager.TextureObserver() {
+              public void textureUpdated(Texture texture) {
+                try {
+                  if (!displayedBackgroundImage.getImage().equals(backgroundImageCache)) {
+                    InputStream contentStream = displayedBackgroundImage.getImage().openStream();
+                    BufferedImage image = ImageIO.read(contentStream);
+                    backgroundImageDimensionCache = new Dimension(image.getWidth(), image.getHeight());
+                    backgroundImageCache = displayedBackgroundImage.getImage();
+                  }
+                  // Update image location and size
+                  float backgroundImageScale = displayedBackgroundImage.getScale();
+                  float imageWidth = backgroundImageScale * backgroundImageDimensionCache.width;
+                  float imageHeight = backgroundImageScale * backgroundImageDimensionCache.height;
+                  Transform3D backgroundImageTransform = new Transform3D();
+                  backgroundImageTransform.setScale(new Vector3d(imageWidth, 1, imageHeight));
+                  backgroundImageTransform.setTranslation(new Vector3d(imageWidth / 2 - displayedBackgroundImage.getXOrigin(), 0f,
+                      imageHeight / 2 - displayedBackgroundImage.getYOrigin()));
+                  backgroundImageAppearance.setTexture(getHomeTextureClone(texture, home));
+                  backgroundImageGroup.setTransform(backgroundImageTransform);
+                  updateGround(waitTextureLoadingEnd,
+                      new Rectangle2D.Float(-displayedBackgroundImage.getXOrigin(), -displayedBackgroundImage.getYOrigin(), imageWidth, imageHeight));
+                } catch (IOException ex) {
+                  // Shouldn't happen since the image was successfully read to create texture
+                  ex.printStackTrace();
+                }
+              }
+            });
+      backgroundImageRenderingAttributes.setVisible(true);
+    } else {
+      backgroundImageRenderingAttributes.setVisible(false);
+      updateGround(waitTextureLoadingEnd, null);
+    }
+  }
+
+  private void updateGround(boolean waitTextureLoadingEnd, Rectangle2D backgroundImageRectangle) {
+    final Home home = (Home)getUserData();
+
     Shape3D groundShape = (Shape3D)getChild(0);
     int currentGeometriesCount = groundShape.numGeometries();
-    
+
     final Appearance groundAppearance = groundShape.getAppearance();
     HomeTexture groundTexture = home.getEnvironment().getGroundTexture();
     if (groundTexture == null) {
       int groundColor = home.getEnvironment().getGroundColor();
       groundAppearance.setMaterial(getMaterial(groundColor, groundColor, 0));
       groundAppearance.setTexture(null);
-      groundAppearance.getTransparencyAttributes().setTransparencyMode(TransparencyAttributes.NONE);      
+      groundAppearance.getTransparencyAttributes().setTransparencyMode(TransparencyAttributes.NONE);
     } else {
       groundAppearance.setMaterial(getMaterial(DEFAULT_COLOR, DEFAULT_COLOR, 0));
       groundAppearance.setTextureAttributes(getTextureAttributes(groundTexture, true));
@@ -125,14 +236,14 @@ public class Ground3D extends Object3DBranch {
               public void textureUpdated(Texture texture) {
                 groundAppearance.setTexture(getHomeTextureClone(texture, home));
                 TransparencyAttributes transparencyAttributes = groundAppearance.getTransparencyAttributes();
-                // If texture isn't transparent, turn off transparency  
-                transparencyAttributes.setTransparencyMode(TextureManager.getInstance().isTextureTransparent(texture) 
+                // If texture isn't transparent, turn off transparency
+                transparencyAttributes.setTransparencyMode(TextureManager.getInstance().isTextureTransparent(texture)
                     ? TransparencyAttributes.NICEST
-                    : TransparencyAttributes.NONE);      
+                    : TransparencyAttributes.NONE);
               }
             });
     }
-    
+
     Area areaRemovedFromGround = new Area();
     // Compute the union of the rooms, the underground walls and furniture areas
     Map<Level, LevelAreas> undergroundLevelAreas = new HashMap<Level, LevelAreas>();
@@ -143,7 +254,7 @@ public class Ground3D extends Object3DBranch {
         float [][] roomPoints = room.getPoints();
         if (roomPoints.length > 2) {
           Area roomArea = new Area(getShape(roomPoints));
-          LevelAreas levelAreas = roomLevel != null && roomLevel.getElevation() < 0 
+          LevelAreas levelAreas = roomLevel != null && roomLevel.getElevation() < 0
               ? getUndergroundAreas(undergroundLevelAreas, roomLevel)
               : null;
           if (roomLevel == null
@@ -160,32 +271,20 @@ public class Ground3D extends Object3DBranch {
         }
       }
     }
-    
-    // Search all items at negative levels that could dig the ground 
-    for (HomePieceOfFurniture piece : home.getFurniture()) {
-      Level pieceLevel = piece.getLevel();
-      if (piece.getGroundElevation() < 0
-          && pieceLevel != null
-          && pieceLevel.isViewable()
-          && pieceLevel.getElevation() < 0) {
-        LevelAreas levelAreas = getUndergroundAreas(undergroundLevelAreas, pieceLevel);
-        if (piece.getStaircaseCutOutShape() == null) {
-          levelAreas.getUndergroundArea().add(new Area(getShape(piece.getPoints())));
-        } else {
-          levelAreas.getUndergroundArea().add(ModelManager.getInstance().getAreaOnFloor(piece));
-        }
-      }
-    }
+
+    // Search all items at negative levels that could dig the ground
+    updateUndergroundAreasDugByFurniture(undergroundLevelAreas, home.getFurniture());
+
     for (Wall wall : home.getWalls()) {
       Level wallLevel = wall.getLevel();
-      if (wallLevel != null 
+      if (wallLevel != null
           && wallLevel.isViewable()
           && wallLevel.getElevation() < 0) {
         LevelAreas levelAreas = getUndergroundAreas(undergroundLevelAreas, wallLevel);
         levelAreas.getWallArea().add(new Area(getShape(wall.getPoints())));
       }
     }
-    // Consider that walls around a closed area define a hole 
+    // Consider that walls around a closed area define a hole
     List<LevelAreas> undergroundAreas = new ArrayList<LevelAreas>(undergroundLevelAreas.values());
     for (LevelAreas levelAreas : undergroundAreas) {
       for (float [][] points : getPoints(levelAreas.getWallArea())) {
@@ -194,7 +293,7 @@ public class Ground3D extends Object3DBranch {
         }
       }
     }
-    
+
     // Sort underground areas in the reverse order of level elevation
     Collections.sort(undergroundAreas, new Comparator<LevelAreas>() {
         public int compare(LevelAreas levelAreas1, LevelAreas levelAreas2) {
@@ -217,13 +316,13 @@ public class Ground3D extends Object3DBranch {
             }
           }
         }
-      }      
+      }
       // Add underground area to ground area at ground level
       for (float [][] points : getPoints(area)) {
         if (new Room(points).isClockwise()) {
-          // Hole surrounded by a union of rooms that form a polygon  
+          // Hole surrounded by a union of rooms that form a polygon
           Area coveredHole = new Area(getShape(points));
-          // Compute the missing hole area in the level area before other sublevels were subtracted from it 
+          // Compute the missing hole area in the level area before other sublevels were subtracted from it
           coveredHole.exclusiveOr(areaAtStart);
           coveredHole.subtract(areaAtStart);
           levelAreas.getUpperLevelArea().add(coveredHole);
@@ -240,10 +339,10 @@ public class Ground3D extends Object3DBranch {
         area.subtract(roomArea);
       }
     }
-    
+
     // Define ground and underground levels surfaces
     Area groundArea = new Area(getShape(new float [][] {
-        {this.originX, this.originY}, 
+        {this.originX, this.originY},
         {this.originX, this.originY + this.depth},
         {this.originX + this.width, this.originY + this.depth},
         {this.originX + this.width, this.originY}}));
@@ -252,21 +351,24 @@ public class Ground3D extends Object3DBranch {
       Area outsideGroundArea = groundArea;
       if (areaRemovedFromGround.isEmpty()) {
         removedAreaBounds = new Rectangle2D.Float(Math.max(-5E3f, this.originX), Math.max(-5E3f, this.originY), 0, 0);
-        removedAreaBounds.add(Math.min(5E3f, this.originX + this.width), 
-            Math.min(5E3f, this.originY + this.depth));            
+        removedAreaBounds.add(Math.min(5E3f, this.originX + this.width),
+            Math.min(5E3f, this.originY + this.depth));
       } else {
-        removedAreaBounds.add(Math.max(removedAreaBounds.getMinX() - 5E3, this.originX), 
+        removedAreaBounds.add(Math.max(removedAreaBounds.getMinX() - 5E3, this.originX),
             Math.max(removedAreaBounds.getMinY() - 5E3, this.originY));
-        removedAreaBounds.add(Math.min(removedAreaBounds.getMaxX() + 5E3, this.originX + this.width), 
+        removedAreaBounds.add(Math.min(removedAreaBounds.getMaxX() + 5E3, this.originX + this.width),
             Math.min(removedAreaBounds.getMaxY() + 5E3, this.originY + this.depth));
       }
       groundArea = new Area(removedAreaBounds);
       outsideGroundArea.subtract(groundArea);
-      // Divide the ground at level 0 in two geometries to limit visual artifacts on large zone  
+      // Divide the ground at level 0 in two geometries to limit visual artifacts on large zone
       addAreaGeometry(groundShape, groundTexture, outsideGroundArea, 0);
     }
     groundArea.subtract(areaRemovedFromGround);
-    // Add level areas for ground level at index 0 because it's the highest level in the list 
+    if (backgroundImageRectangle != null) {
+      groundArea.subtract(new Area(backgroundImageRectangle));
+    }
+    // Add level areas for ground level at index 0 because it's the highest level in the list
     undergroundAreas.add(0, new LevelAreas(new Level("Ground", 0, 0, 0), groundArea));
     float previousLevelElevation = 0;
     for (LevelAreas levelAreas : undergroundAreas) {
@@ -297,7 +399,7 @@ public class Ground3D extends Object3DBranch {
     for (PathIterator it = area.getPathIterator(null, 1); !it.isDone(); it.next()) {
       float [] roomPoint = new float[2];
       if (it.currentSegment(roomPoint) == PathIterator.SEG_CLOSE) {
-        if (areaPartPoints.get(0) [0] == previousRoomPoint [0] 
+        if (areaPartPoints.get(0) [0] == previousRoomPoint [0]
             && areaPartPoints.get(0) [1] == previousRoomPoint [1]) {
           areaPartPoints.remove(areaPartPoints.size() - 1);
         }
@@ -308,12 +410,12 @@ public class Ground3D extends Object3DBranch {
         previousRoomPoint = null;
       } else {
         if (previousRoomPoint == null
-            || roomPoint [0] != previousRoomPoint [0] 
+            || roomPoint [0] != previousRoomPoint [0]
             || roomPoint [1] != previousRoomPoint [1]) {
           areaPartPoints.add(roomPoint);
         }
         previousRoomPoint = roomPoint;
-      }      
+      }
     }
     return areaPoints;
   }
@@ -329,26 +431,51 @@ public class Ground3D extends Object3DBranch {
     return levelAreas;
   }
 
+  /**
+   * Updates underground level areas dug by the visible furniture placed at underground levels.
+   */
+  private void updateUndergroundAreasDugByFurniture(Map<Level, LevelAreas> undergroundLevelAreas, List<HomePieceOfFurniture> furniture) {
+    for (HomePieceOfFurniture piece : furniture) {
+      Level pieceLevel = piece.getLevel();
+      if (piece.getGroundElevation() < 0
+          && piece.isVisible()
+          && pieceLevel != null
+          && pieceLevel.isViewable()
+          && pieceLevel.getElevation() < 0) {
+        if (piece instanceof HomeFurnitureGroup) {
+          updateUndergroundAreasDugByFurniture(undergroundLevelAreas, ((HomeFurnitureGroup)piece).getFurniture());
+        } else {
+          LevelAreas levelAreas = getUndergroundAreas(undergroundLevelAreas, pieceLevel);
+          if (piece.getStaircaseCutOutShape() == null) {
+            levelAreas.getUndergroundArea().add(new Area(getShape(piece.getPoints())));
+          } else {
+            levelAreas.getUndergroundArea().add(ModelManager.getInstance().getAreaOnFloor(piece));
+          }
+        }
+      }
+    }
+  }
+
   /**
    * Adds to ground shape the geometry matching the given area.
    */
-  private void addAreaGeometry(Shape3D groundShape, 
-                               HomeTexture groundTexture, 
+  private void addAreaGeometry(Shape3D groundShape,
+                               HomeTexture groundTexture,
                                Area area, float elevation) {
     List<float [][]> areaPoints = getAreaPoints(area, 1, false);
 
     if (!areaPoints.isEmpty()) {
-      int vertexCount = 0;    
+      int vertexCount = 0;
       int [] stripCounts = new int [areaPoints.size()];
       for (int i = 0; i < stripCounts.length; i++) {
         stripCounts [i] = areaPoints.get(i).length;
         vertexCount += stripCounts [i];
       }
       Point3f [] geometryCoords = new Point3f [vertexCount];
-      TexCoord2f [] geometryTextureCoords = groundTexture != null 
+      TexCoord2f [] geometryTextureCoords = groundTexture != null
           ? new TexCoord2f [vertexCount]
           : null;
-      
+
       int j = 0;
       for (float [][] areaPartPoints : areaPoints) {
         for (int i = 0; i < areaPartPoints.length; i++, j++) {
@@ -359,7 +486,7 @@ public class Ground3D extends Object3DBranch {
           }
         }
       }
-      
+
       GeometryInfo geometryInfo = new GeometryInfo(GeometryInfo.POLYGON_ARRAY);
       geometryInfo.setCoordinates (geometryCoords);
       if (groundTexture != null) {
@@ -375,13 +502,13 @@ public class Ground3D extends Object3DBranch {
   /**
    * Adds to ground shape the geometry matching the given area sides.
    */
-  private void addAreaSidesGeometry(Shape3D groundShape, 
-                                    HomeTexture groundTexture, 
+  private void addAreaSidesGeometry(Shape3D groundShape,
+                                    HomeTexture groundTexture,
                                     float [][] areaPoints,
-                                    float elevation, 
+                                    float elevation,
                                     float sideHeight) {
     Point3f [] geometryCoords = new Point3f [areaPoints.length * 4];
-    TexCoord2f [] geometryTextureCoords = groundTexture != null 
+    TexCoord2f [] geometryTextureCoords = groundTexture != null
         ? new TexCoord2f [geometryCoords.length]
         : null;
     for (int i = 0, j = 0; i < areaPoints.length; i++) {
@@ -409,7 +536,7 @@ public class Ground3D extends Object3DBranch {
     new NormalGenerator(0).generateNormals(geometryInfo);
     groundShape.addGeometry(geometryInfo.getIndexedGeometryArray());
   }
-  
+
   /**
    * Areas of underground levels.
    */
@@ -452,6 +579,6 @@ public class Ground3D extends Object3DBranch {
 
     public Area getUpperLevelArea() {
       return this.upperLevelArea;
-    }    
+    }
   }
 }
