@@ -132,14 +132,14 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
    * Creates the components displayed by this panel.
    */
   private void createComponents(FurnitureCatalog catalog,
-                                final UserPreferences preferences, 
+                                final UserPreferences preferences,
                                 final FurnitureCatalogController controller) {
     final FurnitureCatalogListModel catalogListModel = new FurnitureCatalogListModel(catalog);
     this.catalogFurnitureList = new JList(catalogListModel) {
         private CatalogItemToolTip toolTip = new CatalogItemToolTip(false, preferences);
         private boolean mousePressed;
         private boolean firstScroll;
-        
+
         {
           addMouseListener(new MouseAdapter() {
             @Override
@@ -154,7 +154,7 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
             }
           });
         }
-        
+
         @Override
         public JToolTip createToolTip() {
           if (this.toolTip.isTipTextComplete()) {
@@ -165,7 +165,7 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
             return this.toolTip;
           }
         }
-      
+
         @Override
         public String getToolTipText(MouseEvent ev) {
           // Return a tooltip for furniture pieces described in the list.
@@ -177,7 +177,7 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
             return null;
           }
         }
-        
+
         @Override
         public void scrollRectToVisible(Rectangle rectangle) {
           if (!this.mousePressed
@@ -204,11 +204,11 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
         public void contentsChanged(ListDataEvent ev) {
           spreadFurnitureIconsAlongListWidth();
         }
-  
+
         public void intervalAdded(ListDataEvent ev) {
           spreadFurnitureIconsAlongListWidth();
         }
-  
+
         public void intervalRemoved(ListDataEvent ev) {
           spreadFurnitureIconsAlongListWidth();
         }
@@ -217,13 +217,13 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
         public void ancestorAdded(AncestorEvent ev) {
           spreadFurnitureIconsAlongListWidth();
         }
-  
+
         public void ancestorMoved(AncestorEvent ev) {
           spreadFurnitureIconsAlongListWidth();
         }
-  
+
         public void ancestorRemoved(AncestorEvent ev) {
-        }      
+        }
       });
     addComponentListener(new ComponentAdapter() {
         @Override
@@ -231,12 +231,12 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
           spreadFurnitureIconsAlongListWidth();
         }
       });
-  
+
     updateListSelectedFurniture(catalog, controller);
     addSelectionListeners(catalog, controller);
-    
+
     this.categoryFilterLabel = new JLabel(SwingTools.getLocalizedLabelText(preferences,
-        FurnitureCatalogListPanel.class, "categoryFilterLabel.text"));    
+        FurnitureCatalogListPanel.class, "categoryFilterLabel.text"));
     List<FurnitureCategory> categories = new ArrayList<FurnitureCategory>();
     categories.add(null);
     categories.addAll(catalog.getCategories());
@@ -251,11 +251,11 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
         public Component getListCellRendererComponent(JList list, Object value,
                                                       int index, boolean isSelected, boolean cellHasFocus) {
           if (value == null) {
-            return super.getListCellRendererComponent(list, 
-                preferences.getLocalizedString(FurnitureCatalogListPanel.class, "categoryFilterComboBox.noCategory"), 
+            return super.getListCellRendererComponent(list,
+                preferences.getLocalizedString(FurnitureCatalogListPanel.class, "categoryFilterComboBox.noCategory"),
                 index, isSelected, cellHasFocus);
           } else {
-            return super.getListCellRendererComponent(list, 
+            return super.getListCellRendererComponent(list,
                 ((FurnitureCategory)value).getName(), index, isSelected, cellHasFocus);
           }
         }
@@ -266,26 +266,26 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
           catalogFurnitureList.clearSelection();
         }
       });
-    
+
     this.searchLabel = new JLabel(SwingTools.getLocalizedLabelText(preferences,
         FurnitureCatalogListPanel.class, "searchLabel.text"));
     this.searchTextField = new JTextField(5);
-    this.searchTextField.getDocument().addDocumentListener(new DocumentListener() {  
+    this.searchTextField.getDocument().addDocumentListener(new DocumentListener() {
         public void changedUpdate(DocumentEvent ev) {
           Object selectedValue = catalogFurnitureList.getSelectedValue();
           catalogListModel.setFilterText(searchTextField.getText());
           catalogFurnitureList.clearSelection();
           catalogFurnitureList.setSelectedValue(selectedValue, true);
-          
+
           if (catalogListModel.getSize() == 1) {
             catalogFurnitureList.setSelectedIndex(0);
           }
         }
-  
+
         public void insertUpdate(DocumentEvent ev) {
           changedUpdate(ev);
         }
-  
+
         public void removeUpdate(DocumentEvent ev) {
           changedUpdate(ev);
         }
@@ -298,16 +298,16 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
       });
     if (OperatingSystem.isMacOSXLeopardOrSuperior()) {
       this.searchTextField.putClientProperty("JTextField.variant", "search");
-    } 
-    
+    }
+
     PreferencesChangeListener preferencesChangeListener = new PreferencesChangeListener(this);
     preferences.addPropertyChangeListener(UserPreferences.Property.LANGUAGE, preferencesChangeListener);
     catalog.addFurnitureListener(preferencesChangeListener);
   }
-  
+
   /**
    * Language and catalog listener bound to this component with a weak reference to avoid
-   * strong link between preferences and this component.  
+   * strong link between preferences and this component.
    */
   private static class PreferencesChangeListener implements PropertyChangeListener, CollectionListener<CatalogPieceOfFurniture> {
     private final WeakReference<FurnitureCatalogListPanel> furnitureCatalogPanel;
@@ -331,7 +331,7 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
         // Categories listed in combo box are updated through collectionChanged
       }
     }
-    
+
     public void collectionChanged(CollectionEvent<CatalogPieceOfFurniture> ev) {
       // If panel was garbage collected, remove this listener from catalog
       FurnitureCatalogListPanel furnitureCatalogPanel = this.furnitureCatalogPanel.get();
@@ -339,7 +339,7 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
       if (furnitureCatalogPanel == null) {
         catalog.removeFurnitureListener(this);
       } else {
-        DefaultComboBoxModel model = 
+        DefaultComboBoxModel model =
             (DefaultComboBoxModel)furnitureCatalogPanel.categoryFilterComboBox.getModel();
         FurnitureCategory category = ev.getItem().getCategory();
         List<FurnitureCategory> categories = catalog.getCategories();
@@ -354,13 +354,13 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
   }
 
   /**
-   * Adds mouse listeners that will select only the piece under mouse cursor in the furniture list 
+   * Adds mouse listeners that will select only the piece under mouse cursor in the furniture list
    * before the start of a drag operation, ensuring only one piece can be dragged at a time.
    */
   private void addDragListener(final JList catalogFurnitureList) {
     MouseInputAdapter mouseListener = new MouseInputAdapter() {
         private CatalogPieceOfFurniture exportedPiece;
-  
+
         @Override
         public void mousePressed(MouseEvent ev) {
           this.exportedPiece = null;
@@ -373,7 +373,7 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
             }
           }
         }
-        
+
         public void mouseDragged(MouseEvent ev) {
           if (this.exportedPiece != null) {
             if (catalogFurnitureList.getSelectedIndices().length > 1) {
@@ -387,13 +387,13 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
           }
         }
       };
-    
+
     catalogFurnitureList.addMouseListener(mouseListener);
     catalogFurnitureList.addMouseMotionListener(mouseListener);
   }
 
   /**
-   * Adds mouse listeners to the furniture list to modify selected furniture 
+   * Adds mouse listeners to the furniture list to modify selected furniture
    * and manage links in piece information.
    */
   private void addMouseListeners(final JList catalogFurnitureList,
@@ -416,11 +416,11 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
             }
           }
         }
-        
+
         @Override
         public void mouseMoved(MouseEvent ev) {
           final URL url = getURLAt(ev.getPoint(), catalogFurnitureList);
-          EventQueue.invokeLater(new Runnable() {                  
+          EventQueue.invokeLater(new Runnable() {
               public void run() {
                 if (url != null) {
                   setCursor(handCursor);
@@ -441,8 +441,8 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
                   getListCellRendererComponent(list, piece, pieceIndex, list.isSelectedIndex(pieceIndex), false);
               for (JEditorPane pane : SwingTools.findChildren(rendererComponent, JEditorPane.class)) {
                 Rectangle cellBounds = list.getCellBounds(pieceIndex, pieceIndex);
-                point.x -= cellBounds.x; 
-                point.y -= cellBounds.y + pane.getY(); 
+                point.x -= cellBounds.x;
+                point.y -= cellBounds.y + pane.getY();
                 if (point.x > 0 && point.y > 0) {
                   // Search in information pane if point is over a HTML link
                   int position = pane.viewToModel(point);
@@ -492,34 +492,35 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
    * Layouts the components displayed by this panel.
    */
   private void layoutComponents() {
-    int labelAlignment = OperatingSystem.isMacOSX() 
+    int labelAlignment = OperatingSystem.isMacOSX()
         ? GridBagConstraints.LINE_END
         : GridBagConstraints.LINE_START;
+    int standardGap = Math.round(5 * SwingTools.getResolutionScale());
     // First row
-    Insets labelInsets = new Insets(0, 2, 5, 3);
-    Insets componentInsets = new Insets(0, 2, 3, 0);
+    Insets labelInsets = new Insets(0, 2, standardGap, 3);
+    Insets componentInsets = new Insets(0, 2, Math.round(3 * SwingTools.getResolutionScale()), 0);
     if (!OperatingSystem.isMacOSX()) {
       labelInsets.top = 2;
       componentInsets.top = 2;
       componentInsets.right = 2;
     }
     add(this.categoryFilterLabel, new GridBagConstraints(
-        0, 0, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+        0, 0, 1, 1, 0, 0, GridBagConstraints.LINE_START,
         GridBagConstraints.HORIZONTAL, labelInsets, 0, 0));
     add(this.categoryFilterComboBox, new GridBagConstraints(
-        1, 0, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+        1, 0, 1, 1, 0, 0, GridBagConstraints.LINE_START,
         GridBagConstraints.HORIZONTAL, componentInsets, 0, 0));
     // Second row
     if (OperatingSystem.isMacOSXLeopardOrSuperior()) {
       add(this.searchTextField, new GridBagConstraints(
-          0, 1, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
-          GridBagConstraints.HORIZONTAL, new Insets(0, 0, 3, 0), 0, 0));
-    } else { 
+          0, 1, 2, 1, 0, 0, GridBagConstraints.LINE_START,
+          GridBagConstraints.HORIZONTAL, new Insets(0, 0, Math.round(3 * SwingTools.getResolutionScale()), 0), 0, 0));
+    } else {
       add(this.searchLabel, new GridBagConstraints(
-          0, 1, 1, 1, 0, 0, labelAlignment, 
+          0, 1, 1, 1, 0, 0, labelAlignment,
           GridBagConstraints.NONE, labelInsets, 0, 0));
       add(this.searchTextField, new GridBagConstraints(
-          1, 1, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+          1, 1, 1, 1, 0, 0, GridBagConstraints.LINE_START,
           GridBagConstraints.HORIZONTAL, componentInsets, 0, 0));
     }
     // Last row
@@ -528,7 +529,7 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
         SwingTools.createAdjustmentListenerUpdatingScrollPaneViewToolTip(listScrollPane));
     if (OperatingSystem.isMacOSXHighSierraOrSuperior()
         && !OperatingSystem.isJavaVersionGreaterOrEqual("1.7")) {
-      // Add missing repaint calls on viewport when scroll bar is moved 
+      // Add missing repaint calls on viewport when scroll bar is moved
       listScrollPane.getVerticalScrollBar().addAdjustmentListener(new AdjustmentListener() {
           public void adjustmentValueChanged(AdjustmentEvent ev) {
             listScrollPane.getViewport().repaint();
@@ -537,18 +538,18 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
     }
     listScrollPane.setPreferredSize(new Dimension(250, 250));
     listScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
-    add(listScrollPane, 
+    add(listScrollPane,
         new GridBagConstraints(
-        0, 2, 2, 1, 1, 1, GridBagConstraints.CENTER, 
+        0, 2, 2, 1, 1, 1, GridBagConstraints.CENTER,
         GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
-    
+
     setFocusTraversalPolicyProvider(true);
     setFocusTraversalPolicy(new LayoutFocusTraversalPolicy() {
         @Override
         public Component getDefaultComponent(Container aContainer) {
             EventQueue.invokeLater(new Runnable() {
               public void run() {
-                // Return furniture list only at the first request  
+                // Return furniture list only at the first request
                 setFocusTraversalPolicyProvider(false);
               }
             });
@@ -558,8 +559,8 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
   }
 
   /**
-   * Computes furniture list visible row count to ensure its horizontal scrollbar 
-   * won't be seen. 
+   * Computes furniture list visible row count to ensure its horizontal scrollbar
+   * won't be seen.
    */
   private void spreadFurnitureIconsAlongListWidth() {
     ListModel model = this.catalogFurnitureList.getModel();
@@ -570,54 +571,54 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
     int maxCellWidth = 1;
     int maxCellHeight = 0;
     for (int i = 0; i < size; i++) {
-      Dimension cellPreferredSize = cellRenderer.getListCellRendererComponent(this.catalogFurnitureList, model.getElementAt(i), 
+      Dimension cellPreferredSize = cellRenderer.getListCellRendererComponent(this.catalogFurnitureList, model.getElementAt(i),
           i, this.catalogFurnitureList.isSelectedIndex(i), false).getPreferredSize();
       maxCellWidth = Math.max(maxCellWidth, cellPreferredSize.width);
       maxCellHeight = Math.max(maxCellHeight, cellPreferredSize.height);
     }
-    // Compute a fixed cell width that will spread 
+    // Compute a fixed cell width that will spread
     int visibleItemsPerRow = Math.max(1, extentWidth / maxCellWidth);
-    this.catalogFurnitureList.setVisibleRowCount(size % visibleItemsPerRow == 0 
-        ? size / visibleItemsPerRow 
+    this.catalogFurnitureList.setVisibleRowCount(size % visibleItemsPerRow == 0
+        ? size / visibleItemsPerRow
         : size / visibleItemsPerRow + 1);
     this.catalogFurnitureList.setFixedCellWidth(maxCellWidth + (extentWidth % maxCellWidth) / visibleItemsPerRow);
-    // Set also cell height otherwise first calls to repaint done by icon manager won't repaint it 
-    // because the list have a null size at the beginning  
+    // Set also cell height otherwise first calls to repaint done by icon manager won't repaint it
+    // because the list have a null size at the beginning
     this.catalogFurnitureList.setFixedCellHeight(maxCellHeight);
   }
-  
-  /** 
-   * Adds the listeners that manage selection synchronization in this tree. 
+
+  /**
+   * Adds the listeners that manage selection synchronization in this tree.
    */
-  private void addSelectionListeners(final FurnitureCatalog catalog, 
+  private void addSelectionListeners(final FurnitureCatalog catalog,
                                      final FurnitureCatalogController controller) {
     final SelectionListener modelSelectionListener = new SelectionListener() {
         public void selectionChanged(SelectionEvent selectionEvent) {
-          updateListSelectedFurniture(catalog, controller);        
+          updateListSelectedFurniture(catalog, controller);
         }
       };
     this.listSelectionListener = new ListSelectionListener() {
-        public void valueChanged(ListSelectionEvent ev) {          
+        public void valueChanged(ListSelectionEvent ev) {
           // Updates selected furniture in catalog from selected nodes in tree.
           controller.removeSelectionListener(modelSelectionListener);
           controller.setSelectedFurniture(getSelectedFurniture());
           controller.addSelectionListener(modelSelectionListener);
         }
       };
-      
+
     controller.addSelectionListener(modelSelectionListener);
     this.catalogFurnitureList.getSelectionModel().addListSelectionListener(this.listSelectionListener);
   }
-  
+
   /**
-   * Updates selected items in list from <code>controller</code> selected furniture. 
+   * Updates selected items in list from <code>controller</code> selected furniture.
    */
   private void updateListSelectedFurniture(FurnitureCatalog catalog,
                                            FurnitureCatalogController controller) {
     if (this.listSelectionListener != null) {
       this.catalogFurnitureList.getSelectionModel().removeListSelectionListener(this.listSelectionListener);
     }
-    
+
     this.catalogFurnitureList.clearSelection();
     List<CatalogPieceOfFurniture> selectedFurniture = controller.getSelectedFurniture();
     if (selectedFurniture.size() > 0) {
@@ -627,7 +628,7 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
         for (int i = 0, n = model.getSize(); i < n; i++) {
           if (piece == model.getElementAt(i)) {
             selectedIndices.add(i);
-            break;          
+            break;
           }
         }
       }
@@ -640,7 +641,7 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
         this.catalogFurnitureList.ensureIndexIsVisible(indices [0]);
       }
     }
-    
+
     if (this.listSelectionListener != null) {
       this.catalogFurnitureList.getSelectionModel().addListSelectionListener(this.listSelectionListener);
     }
@@ -655,7 +656,7 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
     System.arraycopy(selectedValues, 0, selectedFurniture, 0, selectedValues.length);
     return Arrays.asList(selectedFurniture);
   }
-  
+
   /**
    * Sets the transfer handler of the list displayed by this panel.
    */
@@ -671,7 +672,7 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
   public TransferHandler getTransferHandler() {
     return this.catalogFurnitureList.getTransferHandler();
   }
-  
+
   /**
    * Sets the popup menu of the list displayed by this panel.
    */
@@ -679,7 +680,7 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
   public void setComponentPopupMenu(JPopupMenu popup) {
     this.catalogFurnitureList.setComponentPopupMenu(popup);
   }
-  
+
   /**
    * Returns the popup menu of the list displayed by this panel.
    */
@@ -687,7 +688,7 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
   public JPopupMenu getComponentPopupMenu() {
     return this.catalogFurnitureList.getComponentPopupMenu();
   }
-  
+
   /**
    * Cell renderer for the furniture list.
    */
@@ -697,7 +698,7 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
     private Font                    modifiablePieceFont;
     private DefaultListCellRenderer nameLabel;
     private JEditorPane             informationPane;
-    
+
     public CatalogCellRenderer() {
       setLayout(null);
       this.nameLabel = new DefaultListCellRenderer() {
@@ -714,31 +715,31 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
       this.defaultFont = UIManager.getFont("ToolTip.font");
       this.modifiablePieceFont = new Font(this.defaultFont.getFontName(), Font.ITALIC, this.defaultFont.getSize());
       this.nameLabel.setFont(this.defaultFont);
-      
+
       this.informationPane = new JEditorPane("text/html", "-");
       this.informationPane.setOpaque(false);
       this.informationPane.setEditable(false);
-      String bodyRule = "body { font-family: " + this.defaultFont.getFamily() + "; " 
-          + "font-size: " + this.defaultFont.getSize() + "pt; " 
+      String bodyRule = "body { font-family: " + this.defaultFont.getFamily() + "; "
+          + "font-size: " + this.defaultFont.getSize() + "pt; "
           + "text-align: center; }";
       ((HTMLDocument)this.informationPane.getDocument()).getStyleSheet().addRule(bodyRule);
-      
+
       add(this.nameLabel);
       add(this.informationPane);
     }
-    
+
     public Component getListCellRendererComponent(JList list,
                                                   Object value,
                                                   int index,
                                                   boolean isSelected,
                                                   boolean cellHasFocus) {
       CatalogPieceOfFurniture piece = (CatalogPieceOfFurniture)value;
-      // Configure name label with its icon, background and focus colors 
-      this.nameLabel.getListCellRendererComponent(list, 
+      // Configure name label with its icon, background and focus colors
+      this.nameLabel.getListCellRendererComponent(list,
           value, index, isSelected, cellHasFocus);
       this.nameLabel.setText(" " + piece.getName() + " ");
       this.nameLabel.setIcon(getLabelIcon(list, piece.getIcon()));
-      this.nameLabel.setFont(piece.isModifiable() 
+      this.nameLabel.setFont(piece.isModifiable()
           ? this.modifiablePieceFont : this.defaultFont);
 
       this.informationPane.setText(piece.getInformation());
@@ -752,55 +753,55 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
       this.informationPane.setBounds(0, namePreferredSize.height,
           getWidth(), getHeight() - namePreferredSize.height);
     }
-    
+
     @Override
     public Dimension getPreferredSize() {
       Dimension preferredSize = this.nameLabel.getPreferredSize();
       preferredSize.height += this.informationPane.getPreferredSize().height + 2;
       return preferredSize;
     }
-    
+
     /**
      * The following methods are overridden for performance reasons.
      */
     @Override
-    public void revalidate() {      
-    }
-    
-    @Override
-    public void repaint(long tm, int x, int y, int width, int height) {      
+    public void revalidate() {
     }
 
     @Override
-    public void repaint(Rectangle r) {      
+    public void repaint(long tm, int x, int y, int width, int height) {
     }
 
     @Override
-    public void repaint() {      
+    public void repaint(Rectangle r) {
+    }
+
+    @Override
+    public void repaint() {
     }
 
     private Icon getLabelIcon(JList list, Content content) {
       return IconManager.getInstance().getIcon(content, DEFAULT_ICON_HEIGHT, list);
     }
-    
+
     @Override
     protected void paintChildren(Graphics g) {
       // Force text anti aliasing on texts
-      ((Graphics2D)g).setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, 
+      ((Graphics2D)g).setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
           RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
       super.paintChildren(g);
     }
   }
-  
+
   /**
-   * List model adaptor to CatalogPieceOfFurniture instances of catalog.  
+   * List model adaptor to CatalogPieceOfFurniture instances of catalog.
    */
   private static class FurnitureCatalogListModel extends AbstractListModel {
     private FurnitureCatalog                catalog;
     private List<CatalogPieceOfFurniture>   furniture;
     private FurnitureCategory               filterCategory;
     private String                          filterText;
-    
+
     public FurnitureCatalogListModel(FurnitureCatalog catalog) {
       this.catalog = catalog;
       this.filterText = "";
@@ -826,7 +827,7 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
       checkFurnitureList();
       return this.furniture.size();
     }
-    
+
     private void resetFurnitureList() {
       if (this.furniture != null) {
         this.furniture = null;
@@ -854,10 +855,10 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
         Collections.sort(this.furniture);
       }
     }
-    
+
     /**
      * Catalog furniture listener bound to this list model with a weak reference to avoid
-     * strong link between catalog and this list.  
+     * strong link between catalog and this list.
      */
     private static class FurnitureCatalogListener implements CollectionListener<CatalogPieceOfFurniture> {
       private WeakReference<FurnitureCatalogListModel>  listModel;
@@ -865,7 +866,7 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
       public FurnitureCatalogListener(FurnitureCatalogListModel catalogListModel) {
         this.listModel = new WeakReference<FurnitureCatalogListModel>(catalogListModel);
       }
-      
+
       public void collectionChanged(CollectionEvent<CatalogPieceOfFurniture> ev) {
         // If catalog list model was garbage collected, remove this listener from catalog
         FurnitureCatalogListModel listModel = this.listModel.get();
