@@ -21,6 +21,7 @@ package com.eteks.sweethome3d.viewcontroller;
 
 import java.beans.PropertyChangeListener;
 import java.beans.PropertyChangeSupport;
+import java.util.Arrays;
 import java.util.List;
 
 import javax.swing.undo.AbstractUndoableEdit;
@@ -41,10 +42,10 @@ import com.eteks.sweethome3d.model.UserPreferences;
  */
 public class PolylineController implements Controller {
   /**
-   * The properties that may be edited by the view associated to this controller. 
+   * The properties that may be edited by the view associated to this controller.
    */
-  public enum Property {THICKNESS, CAP_STYLE, JOIN_STYLE, DASH_STYLE, START_ARROW_STYLE, END_ARROW_STYLE, COLOR}
-  
+  public enum Property {THICKNESS, CAP_STYLE, JOIN_STYLE, DASH_STYLE, DASH_OFFSET, START_ARROW_STYLE, END_ARROW_STYLE, COLOR, ELEVATION}
+
   private final Home                  home;
   private final UserPreferences       preferences;
   private final ViewFactory           viewFactory;
@@ -58,25 +59,29 @@ public class PolylineController implements Controller {
   private Polyline.JoinStyle  joinStyle;
   private boolean             joinStyleEditable;
   private Polyline.DashStyle  dashStyle;
+  private float []            dashPattern;
+  private Float               dashOffset;
   private boolean             arrowsStyleEditable;
   private Polyline.ArrowStyle startArrowStyle;
   private Polyline.ArrowStyle endArrowStyle;
   private Integer             color;
+  private Float               elevation;
+  private Boolean             elevationEnabled;
 
   /**
    * Creates the controller of polyline view with undo support.
    */
-  public PolylineController(final Home home, 
+  public PolylineController(final Home home,
                             UserPreferences preferences,
-                            ViewFactory viewFactory, 
-                            ContentManager contentManager, 
+                            ViewFactory viewFactory,
+                            ContentManager contentManager,
                             UndoableEditSupport undoSupport) {
     this.home = home;
     this.preferences = preferences;
     this.viewFactory = viewFactory;
     this.undoSupport = undoSupport;
     this.propertyChangeSupport = new PropertyChangeSupport(this);
-    
+
     updateProperties();
   }
 
@@ -86,7 +91,7 @@ public class PolylineController implements Controller {
   public DialogView getView() {
     // Create view lazily only once it's needed
     if (this.polylineView == null) {
-      this.polylineView = this.viewFactory.createPolylineView(this.preferences, this); 
+      this.polylineView = this.viewFactory.createPolylineView(this.preferences, this);
     }
     return this.polylineView;
   }
@@ -124,26 +129,27 @@ public class PolylineController implements Controller {
       this.joinStyleEditable = false;
       setJoinStyle(null);
       setDashStyle(null);
+      this.dashPattern = null;
+      setDashOffset(null);
       this.arrowsStyleEditable = false;
       setStartArrowStyle(null);
       setEndArrowStyle(null);
       setColor(null);
+      this.elevationEnabled = Boolean.FALSE;
     } else {
       // Search the common properties among selected polylines
       Polyline firstPolyline = selectedPolylines.get(0);
 
       // Search the common thickness among polylines
       Float thickness = firstPolyline.getThickness();
-      if (thickness != null) {
-        for (int i = 1; i < selectedPolylines.size(); i++) {
-          if (thickness != selectedPolylines.get(i).getThickness()) {
-            thickness = null;
-            break;
-          }
+      for (int i = 1; i < selectedPolylines.size(); i++) {
+        if (thickness != selectedPolylines.get(i).getThickness()) {
+          thickness = null;
+          break;
         }
       }
       setThickness(thickness);
-      
+
       this.capStyleEditable = false;
       for (int i = 0; i < selectedPolylines.size(); i++) {
         if (!selectedPolylines.get(i).isClosedPath()) {
@@ -165,8 +171,8 @@ public class PolylineController implements Controller {
         setCapStyle(capStyle);
       } else {
         setCapStyle(null);
-      }      
-      
+      }
+
       this.joinStyleEditable = false;
       for (int i = 0; i < selectedPolylines.size(); i++) {
         if (selectedPolylines.get(i).getPointCount() > 2) {
@@ -189,17 +195,44 @@ public class PolylineController implements Controller {
       } else {
         setJoinStyle(null);
       }
-      
-      Polyline.DashStyle dashStyle = firstPolyline.getDashStyle();
-      if (dashStyle != null) {
+
+      float [] dashPattern = firstPolyline.getDashPattern();
+      if (dashPattern != null) {
         for (int i = 1; i < selectedPolylines.size(); i++) {
-          if (dashStyle != selectedPolylines.get(i).getDashStyle()) {
-            dashStyle = null;
+          if (!Arrays.equals(dashPattern, selectedPolylines.get(i).getDashPattern())) {
+            dashPattern = null;
             break;
           }
         }
       }
-      setDashStyle(dashStyle);
+      this.dashPattern = dashPattern;
+
+      if (dashPattern != null) {
+        Polyline.DashStyle dashStyle = firstPolyline.getDashStyle();
+        if (dashStyle != null) {
+          for (int i = 1; i < selectedPolylines.size(); i++) {
+            if (dashStyle != selectedPolylines.get(i).getDashStyle()) {
+              dashStyle = null;
+              break;
+            }
+          }
+        }
+        if (dashStyle == Polyline.DashStyle.CUSTOMIZED) {
+        }
+        setDashStyle(dashStyle);
+      } else {
+        setDashStyle(null);
+      }
+
+      // Search the common dash offset among polylines
+      Float dashOffset = firstPolyline.getDashOffset();
+      for (int i = 1; i < selectedPolylines.size(); i++) {
+        if (dashOffset != selectedPolylines.get(i).getDashOffset()) {
+          dashOffset = null;
+          break;
+        }
+      }
+      setDashOffset(dashOffset);
 
       this.arrowsStyleEditable = this.capStyleEditable;
       if (this.arrowsStyleEditable) {
@@ -213,7 +246,7 @@ public class PolylineController implements Controller {
           }
         }
         setStartArrowStyle(startArrowStyle);
-  
+
         Polyline.ArrowStyle endArrowStyle = firstPolyline.getEndArrowStyle();
         if (endArrowStyle != null) {
           for (int i = 1; i < selectedPolylines.size(); i++) {
@@ -240,9 +273,27 @@ public class PolylineController implements Controller {
         }
       }
       setColor(color);
+
+      Float elevation = firstPolyline.getElevation();
+      for (int i = 1; i < selectedPolylines.size(); i++) {
+        if (elevation != selectedPolylines.get(i).getElevation()) {
+          elevation = null;
+          break;
+        }
+      }
+      setElevation(elevation);
+
+      Boolean elevationEnabled = firstPolyline.isVisibleIn3D();
+      for (int i = 1; i < selectedPolylines.size(); i++) {
+        if (!elevationEnabled.equals(selectedPolylines.get(i).isVisibleIn3D())) {
+          elevationEnabled = null;
+          break;
+        }
+      }
+      this.elevationEnabled = elevationEnabled;
     }
   }
-  
+
   /**
    * Sets the edited thickness.
    */
@@ -260,7 +311,7 @@ public class PolylineController implements Controller {
   public Float getThickness() {
     return this.thickness;
   }
-  
+
   /**
    * Sets the edited capStyle.
    */
@@ -278,14 +329,14 @@ public class PolylineController implements Controller {
   public Polyline.CapStyle getCapStyle() {
     return this.capStyle;
   }
-  
+
   /**
    * Returns <code>true</code> if cap style is editable.
    */
   public boolean isCapStyleEditable() {
     return this.capStyleEditable;
   }
-  
+
   /**
    * Sets the edited joinStyle.
    */
@@ -303,16 +354,16 @@ public class PolylineController implements Controller {
   public Polyline.JoinStyle getJoinStyle() {
     return this.joinStyle;
   }
-  
+
   /**
    * Returns <code>true</code> if join style is editable.
    */
   public boolean isJoinStyleEditable() {
     return this.joinStyleEditable;
   }
-  
+
   /**
-   * Sets the edited dashStyle.
+   * Sets the edited dash style.
    */
   public void setDashStyle(Polyline.DashStyle dashStyle) {
     if (dashStyle != this.dashStyle) {
@@ -323,12 +374,37 @@ public class PolylineController implements Controller {
   }
 
   /**
-   * Returns the edited dashStyle.
+   * Returns the edited dash style.
    */
   public Polyline.DashStyle getDashStyle() {
     return this.dashStyle;
   }
-  
+
+  /**
+   * Returns the edited dash pattern.
+   */
+  public float [] getDashPattern() {
+    return this.dashPattern;
+  }
+
+  /**
+   * Sets the edited dash offset.
+   */
+  public void setDashOffset(Float dashOffset) {
+    if (dashOffset != this.dashOffset) {
+      Float oldDashOffset = this.dashOffset;
+      this.dashOffset = dashOffset;
+      this.propertyChangeSupport.firePropertyChange(Property.DASH_OFFSET.name(), oldDashOffset, dashOffset);
+    }
+  }
+
+  /**
+   * Returns the edited dash offset.
+   */
+  public Float getDashOffset() {
+    return this.dashOffset;
+  }
+
   /**
    * Sets the edited start arrow style.
    */
@@ -339,14 +415,14 @@ public class PolylineController implements Controller {
       this.propertyChangeSupport.firePropertyChange(Property.START_ARROW_STYLE.name(), oldStartArrowStyle, startArrowStyle);
     }
   }
-  
+
   /**
    * Returns the edited start arrow style.
    */
   public Polyline.ArrowStyle getStartArrowStyle() {
     return this.startArrowStyle;
   }
-  
+
   /**
    * Sets the edited end arrow style.
    */
@@ -357,21 +433,21 @@ public class PolylineController implements Controller {
       this.propertyChangeSupport.firePropertyChange(Property.END_ARROW_STYLE.name(), oldEndArrowStyle, endArrowStyle);
     }
   }
-  
+
   /**
    * Returns the edited end arrow style.
    */
   public Polyline.ArrowStyle getEndArrowStyle() {
     return this.endArrowStyle;
   }
-  
+
   /**
    * Returns <code>true</code> if arrows style is editable.
    */
   public boolean isArrowsStyleEditable() {
     return this.arrowsStyleEditable;
   }
-  
+
   /**
    * Sets the edited color.
    */
@@ -389,34 +465,65 @@ public class PolylineController implements Controller {
   public Integer getColor() {
     return this.color;
   }
-  
+
+  /**
+   * Sets the edited elevation.
+   */
+  public void setElevation(Float elevation) {
+    if (elevation != this.elevation) {
+      Float oldElevation = this.elevation;
+      this.elevation = elevation;
+      this.propertyChangeSupport.firePropertyChange(Property.ELEVATION.name(), oldElevation, elevation);
+    }
+    this.elevationEnabled = elevation != null;
+  }
+
+  /**
+   * Returns the edited elevation.
+   */
+  public Float getElevation() {
+    return this.elevation;
+  }
+
+  /**
+   * Returns <code>Boolean.TRUE</code> if all edited polylines are viewed in 3D,
+   * or <code>Boolean.FALSE</code> if no polyline is viewed in 3D.
+   */
+  public Boolean isElevationEnabled() {
+    return this.elevationEnabled;
+  }
+
   /**
    * Controls the modification of selected polylines in edited home.
    */
   public void modifyPolylines() {
-    List<Selectable> oldSelection = this.home.getSelectedItems(); 
+    List<Selectable> oldSelection = this.home.getSelectedItems();
     List<Polyline> selectedPolylines = Home.getPolylinesSubList(oldSelection);
     if (!selectedPolylines.isEmpty()) {
-      Float thickness = getThickness(); 
+      Float thickness = getThickness();
       Polyline.CapStyle capStyle = getCapStyle();
       Polyline.JoinStyle joinStyle = getJoinStyle();
       Polyline.DashStyle dashStyle = getDashStyle();
-      Polyline.ArrowStyle startArrowStyle = getStartArrowStyle();  
+      Float dashOffset = getDashOffset();
+      Polyline.ArrowStyle startArrowStyle = getStartArrowStyle();
       Polyline.ArrowStyle endArrowStyle = getEndArrowStyle();
       Integer color = getColor();
-      
+      Float elevation = getElevation();
+      Boolean elevationEnabled = isElevationEnabled();
+
       // Create an array of modified polylines with their current properties values
-      ModifiedPolyline [] modifiedPolylines = new ModifiedPolyline [selectedPolylines.size()]; 
+      ModifiedPolyline [] modifiedPolylines = new ModifiedPolyline [selectedPolylines.size()];
       for (int i = 0; i < modifiedPolylines.length; i++) {
         modifiedPolylines [i] = new ModifiedPolyline(selectedPolylines.get(i));
       }
       // Apply modification
-      doModifyPolylines(modifiedPolylines, thickness, capStyle, joinStyle, dashStyle, startArrowStyle, endArrowStyle, color);       
+      doModifyPolylines(modifiedPolylines, thickness,
+          capStyle, joinStyle, dashStyle, dashOffset, startArrowStyle, endArrowStyle, color, elevation, elevationEnabled);
       if (this.undoSupport != null) {
         UndoableEdit undoableEdit = new PolylinesModificationUndoableEdit(
             this.home, this.preferences, oldSelection,
-            modifiedPolylines, thickness, capStyle, joinStyle, dashStyle, 
-            startArrowStyle, endArrowStyle, color);
+            modifiedPolylines, thickness, capStyle, joinStyle, dashStyle, dashOffset,
+            startArrowStyle, endArrowStyle, color, elevation, elevationEnabled);
         this.undoSupport.postEdit(undoableEdit);
       }
     }
@@ -435,21 +542,26 @@ public class PolylineController implements Controller {
     private Polyline.CapStyle         capStyle;
     private Polyline.JoinStyle        joinStyle;
     private Polyline.DashStyle        dashStyle;
-    private Polyline.ArrowStyle       startArrowStyle;  
+    private Float                     dashOffset;
+    private Polyline.ArrowStyle       startArrowStyle;
     private Polyline.ArrowStyle       endArrowStyle;
     private Integer                   color;
+    private final Float               elevation;
+    private final Boolean             elevationEnabled;
 
     private PolylinesModificationUndoableEdit(Home home,
                                           UserPreferences preferences,
                                           List<Selectable> oldSelection,
-                                          ModifiedPolyline [] modifiedPolylines, 
-                                          Float thickness, 
-                                          Polyline.CapStyle capStyle, 
-                                          Polyline.JoinStyle joinStyle, 
-                                          Polyline.DashStyle dashStyle, 
-                                          Polyline.ArrowStyle startArrowStyle,  
+                                          ModifiedPolyline [] modifiedPolylines,
+                                          Float thickness,
+                                          Polyline.CapStyle capStyle,
+                                          Polyline.JoinStyle joinStyle,
+                                          Polyline.DashStyle dashStyle,
+                                          Float dashOffset,
+                                          Polyline.ArrowStyle startArrowStyle,
                                           Polyline.ArrowStyle endArrowStyle,
-                                          Integer color) {
+                                          Integer color,
+                                          Float elevation, Boolean elevationEnabled) {
       this.home = home;
       this.preferences = preferences;
       this.oldSelection = oldSelection;
@@ -458,24 +570,28 @@ public class PolylineController implements Controller {
       this.capStyle = capStyle;
       this.joinStyle = joinStyle;
       this.dashStyle = dashStyle;
+      this.dashOffset = dashOffset;
       this.startArrowStyle = startArrowStyle;
       this.endArrowStyle = endArrowStyle;
       this.color = color;
+      this.elevation = elevation;
+      this.elevationEnabled = elevationEnabled;
     }
 
     @Override
     public void undo() throws CannotUndoException {
       super.undo();
-      undoModifyPolylines(this.modifiedPolylines); 
-      this.home.setSelectedItems(this.oldSelection); 
+      undoModifyPolylines(this.modifiedPolylines);
+      this.home.setSelectedItems(this.oldSelection);
     }
 
     @Override
     public void redo() throws CannotRedoException {
       super.redo();
-      doModifyPolylines(this.modifiedPolylines, this.thickness, 
-          this.capStyle, this.joinStyle, this.dashStyle, this.startArrowStyle, this.endArrowStyle, this.color); 
-      this.home.setSelectedItems(this.oldSelection); 
+      doModifyPolylines(this.modifiedPolylines, this.thickness,
+          this.capStyle, this.joinStyle, this.dashStyle, this.dashOffset, this.startArrowStyle, this.endArrowStyle, this.color,
+          this.elevation, this.elevationEnabled);
+      this.home.setSelectedItems(this.oldSelection);
     }
 
     @Override
@@ -487,11 +603,11 @@ public class PolylineController implements Controller {
   /**
    * Modifies polylines properties with the values in parameter.
    */
-  private static void doModifyPolylines(ModifiedPolyline [] modifiedPolylines, 
-                                        Float thickness, Polyline.CapStyle capStyle, 
-                                        Polyline.JoinStyle joinStyle, Polyline.DashStyle dashStyle, 
+  private static void doModifyPolylines(ModifiedPolyline [] modifiedPolylines,
+                                        Float thickness, Polyline.CapStyle capStyle,
+                                        Polyline.JoinStyle joinStyle, Polyline.DashStyle dashStyle, Float dashOffset,
                                         Polyline.ArrowStyle startArrowStyle, Polyline.ArrowStyle endArrowStyle,
-                                        Integer color) {
+                                        Integer color, Float elevation, Boolean elevationEnabled) {
     for (ModifiedPolyline modifiedPolyline : modifiedPolylines) {
       Polyline polyline = modifiedPolyline.getPolyline();
       if (thickness != null) {
@@ -506,6 +622,9 @@ public class PolylineController implements Controller {
       if (dashStyle != null) {
         polyline.setDashStyle(dashStyle);
       }
+      if (dashOffset != null) {
+        polyline.setDashOffset(polyline.getDashStyle() != Polyline.DashStyle.SOLID ? dashOffset : 0);
+      }
       if (startArrowStyle != null) {
         polyline.setStartArrowStyle(startArrowStyle);
       }
@@ -515,6 +634,14 @@ public class PolylineController implements Controller {
       if (color != null) {
         polyline.setColor(color);
       }
+      if (elevationEnabled != null) {
+        if (Boolean.FALSE.equals(elevationEnabled)) {
+          polyline.setVisibleIn3D(false);
+        } else if (elevation != null) {
+          polyline.setVisibleIn3D(true);
+          polyline.setElevation(elevation);
+        }
+      }
     }
   }
 
@@ -526,7 +653,7 @@ public class PolylineController implements Controller {
       modifiedPolyline.reset();
     }
   }
-  
+
   /**
    * Stores the current properties values of a modified polyline.
    */
@@ -535,34 +662,43 @@ public class PolylineController implements Controller {
     private final float               thickness;
     private final Polyline.CapStyle   capStyle;
     private final Polyline.JoinStyle  joinStyle;
-    private final Polyline.DashStyle  dashStyle;
+    private final float []            dashPattern;
+    private final Float               dashOffset;
     private final Polyline.ArrowStyle startArrowStyle;
     private final Polyline.ArrowStyle endArrowStyle;
     private final int                 color;
+    private final boolean             visibleIn3D;
+    private final float               elevation;
 
     public ModifiedPolyline(Polyline polyline) {
       this.polyline = polyline;
       this.thickness = polyline.getThickness();
       this.capStyle = polyline.getCapStyle();
       this.joinStyle = polyline.getJoinStyle();
-      this.dashStyle = polyline.getDashStyle();
+      this.dashPattern = polyline.getDashPattern();
+      this.dashOffset = polyline.getDashOffset();
       this.startArrowStyle = polyline.getStartArrowStyle();
       this.endArrowStyle = polyline.getEndArrowStyle();
       this.color = polyline.getColor();
+      this.visibleIn3D = polyline.isVisibleIn3D();
+      this.elevation = polyline.getElevation();
     }
 
     public Polyline getPolyline() {
       return this.polyline;
     }
-    
+
     public void reset() {
       this.polyline.setThickness(this.thickness);
       this.polyline.setCapStyle(this.capStyle);
       this.polyline.setJoinStyle(this.joinStyle);
-      this.polyline.setDashStyle(this.dashStyle);
+      this.polyline.setDashPattern(this.dashPattern);
+      this.polyline.setDashOffset(this.dashOffset);
       this.polyline.setStartArrowStyle(this.startArrowStyle);
       this.polyline.setEndArrowStyle(this.endArrowStyle);
       this.polyline.setColor(this.color);
-    }    
+      this.polyline.setVisibleIn3D(this.visibleIn3D);
+      this.polyline.setElevation(this.elevation);
+    }
   }
 }
